#! python

## Forward Time Centered Space
##
## solution to
## \partial_t u = \alpha \partial^2_{xx} u + f(t,x)
## Dirichlet boundary conditions

import numpy
import sys
import os
import glob
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot
import multiprocessing

# Graph of a scalar function
def graphXY(x, y, window, fileName, xStr, yStr, title, showNow):
    # window = [xMin, xMax, yMin, yMax]
    # showNow = True/False
    fig = matplotlib.pyplot.figure()
    matplotlib.pyplot.axis(window)
    matplotlib.pyplot.xlabel(xStr).set_size('xx-large')
    matplotlib.pyplot.ylabel(yStr,rotation = 0).set_size('xx-large')
    matplotlib.pyplot.title(title).set_size('xx-large')
    matplotlib.pyplot.plot(x,y)
    matplotlib.pyplot.savefig(fileName)
    if showNow:
        matplotlib.pyplot.show(block = False)
    matplotlib.pyplot.close()
    return 1

# Parallelize graph creations
def p_graphXY(x, y, window, fileName, xStr, yStr, title):
    pp = multiprocessing.Process(
        target=graphXY,
        args=(x, y, window,fileName, xStr, yStr, title, False))
    pp.start()
    return 1

# Saving to disk
def uSave(fileName,x,t,u):
    numpy.savez_compressed(fileName, x=x, t=t, u=u)
    return 1

# Parallelize savings to disk
def p_uSave(fileName,x,t,u):
    pp = multiprocessing.Process(target=uSave, args=(fileName,x,t,u))
    pp.start()
    return 1

###

# Reads the directory where the parameters file is
try:
    dirName = sys.argv[1]
except IndexError:
    exit('No directory specified!')

# chooses DIRICHLET directory
dirInt = dirName + "/FTCS_D"

# Reads parameters.py
try:
    exec(open(dirName + "/parameters.py").read())
except:
    exit('Problems with the parameters.py file!')

## removes (*.png, *.npz, *.avi) files from dirName/FTCS_D
## creates dirName/FTCS_D if it does not exist
if os.path.exists(dirInt):
    listing = (glob.glob(dirInt+'/*.png')
                   + glob.glob(dirInt+'/*.npz')
                   + glob.glob(dirInt+'/*.avi'))
    for fileName in listing:
        os.remove(fileName)
    del(listing)
else:
    os.makedirs(dirInt)

#
# Let's start!
#

# mesh definition

x, dx = numpy.linspace(xMin, xMax, nx, retstep=True)
dt = 0.49 * (dx**2) / alpha
T = tMax + dt
t = tMin

# initial datum
u = Initial(x)

# other parameters
i = 0
r = 0.49 ###### alpha * dt / dx**2
s = 1. - 2. * r

while (t < T):
    if (t >= ttp[i]):
        fileName = dirInt + '/u' + str(i).zfill(4)
        if (keep_png or keep_avi):
            p_graphXY(x, u, [xMin, xMax, uMin, uMax],
                        fileName + '.png', '$x$', '$u$',
                        '$u$ at time $t =$ ' + (str(round(t,5))).ljust(7,'0'))
        if keep_npz:
            p_uSave(fileName+'.npz',x,t,u)
        i = i + 1
    t = t + dt
    u[0] = Left(t)
    u[-1] = Right(t)
    u[1:-1] = s * u[1:-1] + r * (u[:-2] + u[2:]) + dt * F (t,x[1:-1],u[1:-1])

if keep_avi:
    cmd = ("mencoder mf://"
               + dirInt
               + "/u*.png -mf w=400:h=300:fps=10:type=png -ovc lavc -lavcopts vcodec=mpeg4:mbd=2:trell -oac copy -o "
               + dirInt + "/film.avi")
    os.system(cmd)
    if not(keep_png):
        listing = glob.glob(dirInt+'/*.png')
        for fileName in listing:
            os.remove(fileName)
        del(listing)
